/**
 * FLL - Level 3
 *
 * Project: Byte Dump
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common enumeration types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _byte_dump_common_enumeration_h
#define _byte_dump_common_enumeration_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Byte Dump modes.
 *
 * byte_dump_mode_*_e:
 *   - none:        No mode.
 *   - hexidecimal: Display using hexidecimal notation (base 16).
 *   - duodecimal:  Display using duodecimal notation (base 12).
 *   - octal:       Display using octal notation (base 8).
 *   - binary:      Display using binary notation (base 2).
 *   - decimal:     Display using decimal notation (base 10).
 */
#ifndef _di_byte_dump_modes_e_
  enum {
    byte_dump_mode_none_e = 0,
    byte_dump_mode_hexidecimal_e,
    byte_dump_mode_duodecimal_e,
    byte_dump_mode_octal_e,
    byte_dump_mode_binary_e,
    byte_dump_mode_decimal_e,
  }; // enum
#endif // _di_byte_dump_modes_e_

/**
 * Byte Dump presentations.
 *
 * byte_dump_presentation_*_e:
 *   - none:    No presentation mode.
 *   - normal:  Normal presentation mode.
 *   - simple:  Simple presentation mode.
 *   - classic: Classic presentation mode.
 */
#ifndef _di_byte_dump_presentations_e_
  enum {
    byte_dump_presentation_none_e = 0,
    byte_dump_presentation_normal_e,
    byte_dump_presentation_simple_e,
    byte_dump_presentation_classic_e,
  }; // enum
#endif // _di_byte_dump_presentations_e_

/**
 * The program parameters.
 *
 * byte_dump_parameter_*_e:
 *   - first: First offset byte size.
 *   - last:  Last offset byte size.
 *
 *   - narrow:      Each character in the displyed text will take at least 1 columns.
 *   - placeholder: Display (colored) placeholders to signify codes that are UTF-8 fragments.
 *   - text:        Display text
 *   - wide:        Each character in the displyed text will take at least 2 columns.
 *   - width:       Number of characters to display per row.
 *
 *   - binary:      Display using binary format.
 *   - decimal:     Display using decimal format.
 *   - duodecimal:  Display using duodecimal format.
 *   - hexidecimal: Display using hexidecimal format.
 *   - octal:       Display using octal format.
 *   - unicode:     Display using Unicode format.
 *
 *   - normal:  Use normal presentation, displaying UTF-8 sequence codes for ASCII special codes.
 *   - simple:  Use simple presentation, displaying spaces for ASCII special codes instead of UTF-8 sequence codes.
 *   - classic: Use classic presentation, displaying periods for ASCII special codes instead of UTF-8 sequence codes.
 */
#ifndef _di_byte_dump_parameter_e_
  enum {
    byte_dump_parameter_binary_e = f_console_standard_parameter_last_e,
    byte_dump_parameter_decimal_e,
    byte_dump_parameter_duodecimal_e,
    byte_dump_parameter_hexidecimal_e,
    byte_dump_parameter_octal_e,
    byte_dump_parameter_unicode_e,

    byte_dump_parameter_first_e,
    byte_dump_parameter_last_e,

    byte_dump_parameter_narrow_e,
    byte_dump_parameter_placeholder_e,
    byte_dump_parameter_text_e,
    byte_dump_parameter_wide_e,
    byte_dump_parameter_width_e,

    byte_dump_parameter_normal_e,
    byte_dump_parameter_simple_e,
    byte_dump_parameter_classic_e,
  }; // enum

  #define byte_dump_console_parameter_t_initialize \
    { \
      macro_fll_program_console_parameter_standard_initialize, \
      \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_binary_s,      byte_dump_long_binary_s,      0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_decimal_s,     byte_dump_long_decimal_s,     0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_duodecimal_s,  byte_dump_long_duodecimal_s,  0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_hexidecimal_s, byte_dump_long_hexidecimal_s, 0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_octal_s,       byte_dump_long_octal_s,       0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_unicode_s,     byte_dump_long_unicode_s,     0, f_console_flag_normal_e), \
      \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_first_s,       byte_dump_long_first_s,       1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_last_s,        byte_dump_long_last_s,        1, f_console_flag_normal_e), \
      \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_narrow_s,      byte_dump_long_narrow_s,      0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_placeholder_s, byte_dump_long_placeholder_s, 0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_text_s,        byte_dump_long_text_s,        0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_wide_s,        byte_dump_long_wide_s,        0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(byte_dump_short_width_s,       byte_dump_long_width_s,       1, f_console_flag_normal_e), \
      \
      macro_f_console_parameter_t_initialize_5(byte_dump_long_normal_s,  0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_5(byte_dump_long_simple_s,  0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_5(byte_dump_long_classic_s, 0, f_console_flag_normal_e), \
    }

  #define byte_dump_parameter_total_d (f_console_parameter_state_type_total_d + 16)
#endif // _di_byte_dump_parameter_e_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _byte_dump_common_enumeration_h
